using Jypeli;
using Jypeli.Assets;
// ReSharper disable All


namespace HyttysPeli;

/// @author salmatel
/// @version 12.11.2023
/// <summary>
/// 
/// </summary>
public class HyttysPeli : PhysicsGame
{
    private Image taustaKuva = LoadImage("taustakuva");
    private IntMeter pistelaskuri;
    private IntMeter helttilaskuri;
    private PhysicsObject pelaaja;
    private Image kuvaAmpiainen = LoadImage("ampiainen");
    
    public override void Begin()
    {
        LuoPistelaskuri();
        LuoHelttilaskuri();
        BoundingRectangle[] reunat = LuoKentta();
        LuoPelaaja();
        LuoOhjaimet();
        //Luo aliohjelmat näille, paranna reunat, tee lopetus
        for (int i = 0; i < 25; i++)
        {
            LuoHyttynen(this, reunat[0], 60, "hyttynen");
        }

        for (int j = 0; j < 10; j++)
        {
            LuoAmpiainen(this, reunat[0], 60, "ampiainen");
        }

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }

    private void LuoOhjaimet()
    {
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä avustus");
        Keyboard.Listen(Key.Up, ButtonState.Down, LyoHyttysta, "Pelaaja ylös", pelaaja, new Vector(0, 20));
        Keyboard.Listen(Key.Down, ButtonState.Down, LyoHyttysta, "Pelaaja alas", pelaaja, new Vector(0, -20));
        Keyboard.Listen(Key.Left, ButtonState.Down, LyoHyttysta, "Pelaaja vasemmalle", pelaaja, new Vector(-20, 0));
        Keyboard.Listen(Key.Right, ButtonState.Down, LyoHyttysta, "Pelaaja oikealle", pelaaja, new Vector(20, 0));
    }
    private void LuoPelaaja()
    {
        pelaaja = new PhysicsObject(25, 100, Shape.Rectangle);
        pelaaja.Y = Level.Bottom;
        pelaaja.X = 50;
        pelaaja.Image = LoadImage("hyttyskarkoite");
        Add(pelaaja); 
        AddCollisionHandler(pelaaja, "hyttynen", PelaajaTormasiHyttyseen);
        AddCollisionHandler(pelaaja, "ampiainen", PelaajaTormasiAmpiaiseen);
    }
    private BoundingRectangle[] LuoKentta()
    {
        // Jos halutaan seinien kimpoilut, vaihdetaan seinät Jypelin ohjeisiin
        MasterVolume = 0.05;
        SetWindowSize(1920, 1080, true);
        Level.Background.Image = taustaKuva;
        BoundingRectangle alaosa = new BoundingRectangle(new Vector(Level.Left, 0), Level.BoundingRect.BottomRight);
        BoundingRectangle ylaosa = new BoundingRectangle(Level.BoundingRect.TopLeft, new Vector(Level.Right, 0));
        Level.CreateBorders();
        BoundingRectangle[] reunat = new[] { ylaosa, alaosa };
        return reunat;
    }

    private void HyttynenTormasiSeinaan(PhysicsObject reuna, PhysicsObject hyttynen)
    {
        
    }
    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja hyttynen törmäävät.
    /// Luodaan hyttysen kohdalla räjähdys ja poistetaan hyttynen pelistä
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="hyttynen">hyttynen johon pelaaja törmäsi</param>
    private void PelaajaTormasiHyttyseen(PhysicsObject pelaaja, PhysicsObject hyttynen)
    {
        Explosion rajahdys = new Explosion(hyttynen.Width * 2);
        rajahdys.Position = hyttynen.Position;
        rajahdys.UseShockWave = false; 
        this.Add(rajahdys);
        Remove(hyttynen);
        pistelaskuri.Value += 1;
    }


    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja joku ampiainen törmäävät.
    /// Jos pelaaja osuu ampiaiseen, peli alkaa alusta.
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="ampiainen">ampiainen johon pelaaja törmäsi</param>
    private void PelaajaTormasiAmpiaiseen(PhysicsObject pelaaja, PhysicsObject ampiainen)
    {
        Remove(ampiainen);
        helttilaskuri.Value -= 1;
        if (helttilaskuri.Value == 0)
        {
            ClearAll();
            Begin();
        }
    }
    
    
    /// <summary>
    /// Lyödään hyttystä voimavektorilla
    /// </summary>
    /// <param name="hyttynen">lyötävä hyttynen</param>
    /// <param name="suunta">voimavektori</param>
    public static void LyoHyttysta(PhysicsObject hyttynen, Vector suunta)
    { 
        hyttynen.Hit(hyttynen.Mass * suunta);
    }

    
    /// <summary>
    /// Luodaan satunnainen hyttynen.
    /// </summary>
    /// <param name="peli">Mihin peliin hyttynen luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Hyttysen vauhti alkutilanteessa</param>
    /// <param name="tunniste">Hyttyselle annettava tunniste</param>
    /// <returns>Viite luotuun hyttyseen</returns>
    public PhysicsObject LuoHyttynen(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = 50;
        double korkeus = 50;
        PhysicsObject hyttynen = new PhysicsObject(leveys, korkeus, Shape.Rectangle);
        hyttynen.Position = RandomGen.NextVector(rect);
        Vector suunta = RandomGen.NextVector(0, vauhti + 100);
        hyttynen.Hit(suunta);
        // hyttynen.Velocity = suunta;
        hyttynen.Tag = tunniste;
        hyttynen.Image = LoadImage("hyttynen (2)");
        // AddCollisionHandler(hyttynen, "alaosa", HyttynenTormasiSeinaan);
        peli.Add(hyttynen);
        return hyttynen;
    }
    
    
    /// <summary>
    /// Luodaan satunnainen ampiainen.
    /// </summary>
    /// <param name="peli">Mihin peliin ampiainen luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Ampiaisen vauhti alkutilanteessa</param>
    /// <param name="tunniste">Ampiaiselle annettava tunniste</param>
    /// <returns>Viite luotuun Ampiaiseen</returns>
    public PhysicsObject LuoAmpiainen(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = 75;
        double korkeus = 75;
        PhysicsObject ampiainen = new PhysicsObject(leveys, korkeus, Shape.Rectangle);
        ampiainen.Position = RandomGen.NextVector(rect);
        Vector suunta = RandomGen.NextVector(0, vauhti);
        ampiainen.Hit(suunta);
        ampiainen.Tag = tunniste;
        ampiainen.Image = kuvaAmpiainen;
        peli.Add(ampiainen);
        return ampiainen;
    }
    
    
    void LuoPistelaskuri()
    {
        
        pistelaskuri = new IntMeter(0);               
      
        Label pistenaytto = new Label(); 
        pistenaytto.X = Screen.Left + 100;
        pistenaytto.Y = Screen.Top - 100;
        pistenaytto.TextColor = Color.Black;
        pistenaytto.Color = Color.White;
        pistenaytto.Title = "Pisteitä: ";

        pistenaytto.BindTo(pistelaskuri);
        Add(pistenaytto);
    }
    
    
    void LuoHelttilaskuri()
    {
        helttilaskuri = new IntMeter(3);               
      
        Label helttinaytto = new Label(); 
        helttinaytto.X = Screen.Right - 100;
        helttinaytto.Y = Screen.Top - 100;
        helttinaytto.TextColor = Color.Black;
        helttinaytto.Color = Color.Red;
        helttinaytto.Title = "Elämät: ";

        helttinaytto.BindTo(helttilaskuri);
        Add(helttinaytto);
    }
}


